@@ -3,7 +3,7 @@
 
 # The MIT License (MIT)
 
-# Copyright (c) 2014 CNRS
+# Copyright (c) 2014-2016 CNRS
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -38,10 +38,12 @@ import numpy as np
 from . import PYANNOTE_URI, PYANNOTE_MODALITY, \
     PYANNOTE_SEGMENT, PYANNOTE_TRACK, PYANNOTE_LABEL
 from banyan import SortedDict
+from xarray import DataArray
 from .interval_tree import TimelineUpdator
 from .segment import Segment
 from .timeline import Timeline
 from .json import PYANNOTE_JSON, PYANNOTE_JSON_CONTENT
+from .util import string_generator, int_generator
 
 # ignore Banyan warning
 warnings.filterwarnings(
@@ -202,7 +204,9 @@ class Annotation(object):
 
     def itertracks(self, label=False):
         for segment, tracks in self._tracks.items():
-            for track, lbl in sorted(six.iteritems(tracks)):
+            for track, lbl in sorted(
+                six.iteritems(tracks),
+                key=lambda tl: (str(tl[0]), str(tl[1]))):
                 if label:
                     yield segment, track, lbl
                 else:
@@ -494,20 +498,26 @@ class Annotation(object):
         if isinstance(key, Segment):
             key = (key, '_')
 
+        segment, track = key
+
+        # do not add empty track
+        if not segment:
+            return
+
         # in case we create a new segment
         # mark timeline as modified
-        if key[0] not in self._tracks:
-            self._tracks[key[0]] = {}
+        if segment not in self._tracks:
+            self._tracks[segment] = {}
             self._timelineNeedsUpdate = True
 
         # in case we modify an existing track
         # mark old label as modified
-        if key[1] in self._tracks[key[0]]:
-            old_label = self._tracks[key[0]][key[1]]
+        if track in self._tracks[segment]:
+            old_label = self._tracks[segment][track]
             self._labelNeedsUpdate[old_label] = True
 
         # mark new label as modified
-        self._tracks[key[0]][key[1]] = label
+        self._tracks[segment][track] = label
         self._labelNeedsUpdate[label] = True
 
     def empty(self):
@@ -531,7 +541,7 @@ class Annotation(object):
         if any([lnu for lnu in self._labelNeedsUpdate.values()]):
             self._updateLabels()
 
-        labels = sorted(self._labels)
+        labels = sorted(self._labels, key=str)
 
         if not unknown:
             labels = [l for l in labels if not isinstance(l, Unknown)]
@@ -801,11 +811,14 @@ class Annotation(object):
     def __mod__(self, translation):
         return self.translate(translation)
 
-    def anonymize_labels(self):
+    def anonymize_labels(self, generator='string'):
         """Anonmyize labels
 
-        Create a new annotation where labels are anonymized, ie. each label
-        is replaced by a unique `Unknown` instance.
+        Create a new annotation where labels are anonymized.
+
+        Parameters
+        ----------
+        generator : {'string', 'int', iterator}, optional
 
         Returns
         -------
@@ -813,25 +826,42 @@ class Annotation(object):
             New annotation with anonymized labels.
 
         """
-        translation = {label: Unknown() for label in self.labels()}
-        return self % translation
 
-    def anonymize_tracks(self):
+        if generator == 'string':
+            generator = string_generator()
+        elif generator == 'int':
+            generator = int_generator()
+
+        mapping = {label: next(generator) for label in self.labels()}
+        return self.translate(mapping)
+
+    def anonymize_tracks(self, generator='string'):
         """
         Anonymize tracks
 
-        Create a new annotation where each track is anonymized, i.e. the label
-        of each track is set to a unique `Unknown` instance
+        Create a new annotation where each track has a unique label.
+
+        Parameters
+        ----------
+        generator : {'string', 'int', iterator}, optional
+            Default to 'string'.
 
         Returns
         -------
         anonymized : `Annotation`
-            Anonymized annotation
+            New annotation with anonymized tracks.
 
         """
+
+        if generator == 'string':
+            generator = string_generator()
+        elif generator == 'int':
+            generator = int_generator()
+
         anonymized = self.empty()
         for s, t, _ in self.itertracks(label=True):
-            anonymized[s, t] = Unknown()
+            anonymized[s, t] = next(generator)
+
         return anonymized
 
     def smooth(self, collar=0.):
@@ -892,11 +922,30 @@ class Annotation(object):
         """
 
         for s, S in self.get_timeline().co_iter(other.get_timeline()):
-            tracks = self.get_tracks(s)
-            other_tracks = other.get_tracks(S)
+            tracks = sorted(self.get_tracks(s), key=str)
+            other_tracks = sorted(other.get_tracks(S), key=str)
             for t, T in itertools.product(tracks, other_tracks):
                 yield (s, t), (S, T)
 
+    def __mul__(self, other):
+        """Compute cooccurrence matrix"""
+
+        i = self.labels()
+        j = other.labels()
+
+        matrix = DataArray(
+            np.zeros((len(i), len(j))),
+            coords=[('i', i), ('j', j)])
+
+        for (segment, track), (other_segment, other_track) in self.co_iter(other):
+            label = self[segment, track]
+            other_label = other[other_segment, other_track]
+            duration = (segment & other_segment).duration
+            matrix.loc[label, other_label] += duration
+
+        return matrix
+
+
     def for_json(self):
 
         data = {PYANNOTE_JSON: self.__class__.__name__}
@@ -929,7 +978,7 @@ class Annotation(object):
         return annotation
 
     def _repr_png_(self):
-        from pyannote.core.notebook import repr_annotation
+        from .notebook import repr_annotation
         return repr_annotation(self)
 
 
