@@ -188,6 +188,10 @@ class Scores(object):
 
         segment, track, label = key
 
+        # do not add empty track
+        if not segment:
+            return
+
         self.dataframe_.at[tuple(segment) + (track,), label] = value
         self.annotation_[segment, track] = label
         self.hasChanged_ = True
@@ -612,7 +616,7 @@ class Scores(object):
             return ""
 
     def _repr_png_(self):
-        from pyannote.core.notebook import repr_scores
+        from .notebook import repr_scores
         return repr_scores(self)
 
 
